﻿using System;
using System.Collections.Generic;
namespace Radio
{
    #region Using Directives

    using System.Windows.Forms;

    #endregion Using Directives

    public partial class Form1 : Form
    {
        private enum Colors { None = -1, Red = 0, Green = 1, Blue = 2 }
        private enum Sizes { None = -1, Large = 0, Medium = 1, Small = 2 }

        // NOTE: Sizes are dependent on Color selection
        // Button dependency rules:
        // Red - all sizes are available
        // Green - large and medium is available
        // Blue - small is available

        public Form1( )
        {
            InitializeComponent( );
        }

        private void btnClearG1_Click( object sender, EventArgs e )
        {
            ClearRadioButtonsInGroup( this.groupBox1 );
        }

        private void btnClearG2_Click( object sender, EventArgs e )
        {
            ClearRadioButtonsInGroup( this );
        }

        private void btnOK_Click( object sender, EventArgs e )
        {
            Colors color = (Colors)GetSelectedRadioIndex( this.groupBox1 );
            Sizes size = (Sizes) GetSelectedRadioIndex( this );

            MessageBox.Show( String.Format( "Selected Color: {0}\nSelected Size: {1}", color, size ) );
        }

        private void btnCancel_Click( object sender, EventArgs e )
        {
            Close( );
        }

        private void OnColorCheckedChanged( object sender, EventArgs e )
        {
            EnableSizeRadioButtons( );

            ForceSizeRadioButtonSelection( );
        }

        /// <summary>
        /// Enables the size radio buttons based on the
        /// Color radio button selection
        /// </summary>
        private void EnableSizeRadioButtons( )
        {
            // Enable all Size radio buttons
            this.radioButtonLarge.Enabled = true;
            this.radioButtonMedium.Enabled = true;
            this.radioButtonSmall.Enabled = true;

            // Button dependency rules:
            // Red - all sizes are available
            // Green - large and medium is available
            // Blue - small is available
            switch( (Colors) GetSelectedRadioIndex( groupBox1 ) )
            {
            case Colors.Green:
                this.radioButtonSmall.Enabled = false;
                break;
            case Colors.Blue:
                this.radioButtonLarge.Enabled = false;
                this.radioButtonMedium.Enabled = false;
                break;
            }

        }

        /// <summary>
        /// Forces a size radio button based on selected color
        /// When Color.None or Color.Red is selected, does nothing
        /// When Color.Green is selected, clears Sizes.Small if selected
        /// When Color.Blue is selected, selects Sizes.Small
        /// </summary>
        private void ForceSizeRadioButtonSelection( )
        {
            switch ( ( Colors ) GetSelectedRadioIndex( groupBox1 ) )
            {
            case Colors.Green:
                // Clear previous selection, if user had the
                // now unavailable Small option selected
                if ( Sizes.Small == ( Sizes ) GetSelectedRadioIndex( this ) )
                {
                    SetRadioButtonByIndex( this, NoneSelectedIndex );
                }
                break;
            case Colors.Blue:
                // Since small is the only option, select it
                SetRadioButtonByIndex( this, ( int ) Sizes.Small );
                break;
            }

        }

        #region Radio Button Helper methods

        /// <summary>
        /// Clears all the radio buttons within a group
        /// </summary>
        /// <param name="parent"></param>
        private void ClearRadioButtonsInGroup( Control parent )
        {
            foreach ( Control child in parent.Controls )
            {
                RadioButton radio = child as RadioButton;

                if ( radio != null )
                {
                    radio.Checked = false;
                }
            }
        }

        /// <summary>
        /// Returns a zero-based index of the selected radio
        /// button within the group.
        /// </summary>
        /// <param name="parent">Control which contains the radio buttons</param>
        /// <returns>Index of selected radio button; -1 if none are selected</returns>
        private int GetSelectedRadioIndex( Control parent )
        {
            int index = NoneSelectedIndex;

            var radioButtons = new SortedDictionary< int, RadioButton >( );

            // Add the radio buttons to the dictionary
            // sorted by tab index
            foreach ( Control child in parent.Controls )
            {
                RadioButton radio = child as RadioButton;

                if ( radio != null )
                {
                    radioButtons.Add( radio.TabIndex, radio );
                }
            }

            // Walk through the dictionary (which are 
            // sorted by tab index) to retrieve the index
            // of the selected radio button.
            // NOTE: The first index within this group
            // will always be 0 regardless of the starting
            // tab index.  This is done so you know that
            // 0 always represent the first radio button
            // within the group
            foreach ( int tabIndex in radioButtons.Keys )
            {
                index++;

                if ( radioButtons [ tabIndex ].Checked )
                {
                    return index;
                }
            }

            return NoneSelectedIndex;
        }

        /// <summary>
        /// Sets the radio button within a group by index
        /// </summary>
        /// <param name="parent">Control which contains the radio buttons</param>
        /// <param name="index">zero-based index of radio button to select</param>
        private void SetRadioButtonByIndex( Control parent, int index )
        {
            int indexCurrent = NoneSelectedIndex;

            var radioButtons = new SortedDictionary<int, RadioButton>( );

            // Add the radio buttons to the dictionary
            // sorted by tab index
            foreach ( Control child in parent.Controls )
            {
                RadioButton radio = child as RadioButton;

                if ( radio != null )
                {
                    radioButtons.Add( radio.TabIndex, radio );
                }
            }

            // Walk through the dictionary (which are 
            // sorted by tab index) to retrieve the index
            // of the selected radio button.
            // NOTE: The first index within this group
            // will always be 0 regardless of the starting
            // tab index.  This is done so you know that
            // 0 always represent the first radio button
            // within the group
            foreach ( int tabIndex in radioButtons.Keys )
            {
                indexCurrent++;

                radioButtons [ tabIndex ].Checked = ( indexCurrent == index );
            }
        }

        #endregion Radio Button Helper methods

        private readonly int NoneSelectedIndex = -1;
    }
}
